<?php

declare(strict_types=1);

namespace mirzaev\vk\api\methods;

// Файлы проекта
use mirzaev\vk\robots\robot;

// Встроенные библиотеки
use Exception,
    stdClass;

/**
 * Пользователь
 *
 *
 * @see https://dev.vk.com/method/users
 *
 * @package mirzaev\vk\api
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class users extends method
{
    /**
     * Конструктор
     *
     * @param robot $user Робот
     */
    public function __construct(
        protected robot $robot
    ) {
    }

    /**
     * Запросить информацию о пользователе
     *
     * @see https://dev.vk.com/method/users.get
     *
     * @param array $receiver Выбор пользователей для запроса информации (user_ids)
     * @param array $fields Дополнительные запрашиваемые поля
     * @param string $name_case Падеж
     *
     * @return stdClass|array|null Информация об аккаунте или массив с информацией об аккаунтах, если найдена
     *
     * @todo
     * 1. Доделать
     * 2. Написать обработчик ошибок возвращаемых ВКонтакте
     */
    public function get(int|string|array $receiver = [], array $fields = [], ?string $name_case = null): stdClass|array|null
    {
        // Реиницилазиция
        $this->robot->api->reinit();

        if (is_int($receiver)) {
            // Идентификатор

            // Инициализация пользователя
            $this->robot->api['user_ids'] = $receiver;
        } else if (is_array($receiver)) {
            // Идентификаторы

            // Инициализация пользователей
            $this->robot->api['user_ids'] = '';

            foreach ($receiver as $key => $user_id) {
                // Перебор пользователей для получения информации

                // Запись пользователя
                $this->robot->api['user_ids'] .= $user_id;

                // Запись разделителя
                if ($key === array_key_last($receiver)) break;
                else $this->robot->api['user_ids'] .= ', ';
            }
        } else if (is_string($receiver)) {
            // Домен

            // Инициализация пользователя
            $this->robot->api['user_ids'] = $receiver;
        }

        if (isset($fields)) {
            // Запрошены дополнительные запрашиваемые поля

            // Инициализация дополнительных запрашиваемых полей
            $this->robot->api['fields'] = '';

            foreach ($fields as $key => $field) {
                // Перебор дополнительных запрашиваемых полей

                // Запись запрашиваемого дополнительного поля
                $this->robot->api['fields'] .= $field;

                // Запись разделителя
                if ($key !== array_key_last($fields)) $this->robot->api['fields'] .= ', ';
            }
        }

        // Инициализация падежа
        if (isset($name_case)) $this->robot->api['name_case'] = $name_case;

        // Запрос
        $request = json_decode($this->robot->browser->request('POST', 'users.get', ['form_params' => $this->robot->api->settings])->getBody()->getContents());

        // Если в ответе ошибка
        if (isset($request->error)) {
            throw new Exception('ВКонтакте: ' . $request->error->error_msg, $request->error->error_code);
        }

        return is_array($receiver) ? $request->response : (isset($request->response[0]) ? $request->response[0] : null);
    }
}
